
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>


enum {
	// _xinfreq = 20_000_000,
	_xtlfreq = 20_000_000,
	_clkfreq = 200_000_000,
	DOWNLOAD_BAUD = 230_400,
	DEBUG_BAUD = DOWNLOAD_BAUD,


	SPI_TX_PIN = 8 | 2<<6,    // must be an even pin for differential pairing
	SPI_CS_PIN = 10,

	CLK_DIV = 1,    // sysclock divider for SPI clock gen
	TX_REGD = 1,    // tx data pins:  0 for unregistered, 1 for registered

};


static void  spi_shift_out( uint8_t *paddr, size_t len )
{
    __asm volatile { // no optimising and enforces Fcache use - Needed to free up the FIFO
		rdfast	#0, paddr           // non-blocking FIFO prefetch from haddr (byte granular)
		setxfrq	mnco                // set sysclock/1 for lead in timing
		shl     len, #1
.segment
		setword	mdat, len, #0
		sub	len, len32k   wcz
	if_a	setword	mdat, len32k, #0
		xzero   mdat, #0            // tx data from FIFO
	if_a	jmp	#.segment

	_ret_	waitxfi                     // wait until streamer is done

mnco		long	0x8000_0000UL / CLK_DIV + (0x8000_0000UL % CLK_DIV > 0 ? 1 : 0)    // round up upon non-zero remainder
mdat		long	X_RFBYTE_4P_1DAC4 | (SPI_TX_PIN & 0x3c)<<17 | X_PINS_ON | X_ALT_ON  // X_ALT_ON for big-endian nibble order
len32k		long    0x8000
    } // Exiting of Fcache/Cogexec issues an implicit RDFAST, flushing remaining FIFO content to hubRAM
}


void  filldat( uint8_t *addr, size_t len )
{
    size_t  i, j;
    uint8_t  dat;

    for( j = 0; j < len; j++ )    // erase the streamer pin data
        addr[j] = 0;

    for( j = 1; j < 10000; j++ )    // bytes
    {
        for( i = 0; i <= 7; i++ )    // bits of byte
        {
            dat = (j>>(7 - i)) & 1;    // big-endian bit order
            addr[(j<<3) + i] = 0x46 | dat | (dat<<4);    // CS low, clock low->high, CPHA=0 CPOL=0
//            addr[(j<<3) + i] = 0x64 | dat | (dat<<4);    // CS low, clock high->low, CPHA=1 CPOL=0
        }
    }
    addr[j<<3] = 0x44;    // trailing CS low, returns high after this
}


void  main( void )
{
    uint8_t  txdata[80_100];
    int  i,  blockid = 0;
    uint8_t  dat;

    _pinl( SPI_TX_PIN );
    _wrpin( SPI_TX_PIN, TX_REGD ? P_SYNC_IO : 0 );
    _wrpin( SPI_CS_PIN, P_INVERT_OUTPUT | (TX_REGD ? P_SYNC_IO : 0) );    // CS inverted so that 0 bit is a high pin

    _waitms(200);
    printf( "\n   clkfreq = %d   clkmode = 0x%x\n", _clockfreq(), _clockmode() );

    filldat( txdata, sizeof(txdata) );
/*
    for( size_t j = 0; j < 40; j++ )
        printf( " %x", txdata[j] );
    printf( "\n" );
*/
    while(1)
    {
        for( i = 0; i <= 7; i++ )    // bits of byte
        {
            dat = (blockid>>(7 - i)) & 1;    // big-endian bit order
            txdata[i] = 0x46 | dat | (dat<<4);    // CS low, clock low->high, CPHA=0 CPOL=0
//            txdata[i] = 0x64 | dat | (dat<<4);    // CS low, clock high->low, CPHA=1 CPOL=0
        }
        blockid = (blockid + 1) % 20;

        printf( " Byte count = %d\n", sizeof(txdata) );
        spi_shift_out( txdata, sizeof(txdata) );
        _waitms( 2000 );
    }
}
